# Database structure #

## Tables ##

Every table of the database has the following structure

* `ID : int` - Unique ID for the resource
* `Date : timestamp` - Date when the resource was created
* Public data - Data stored in plain text
    * ...
* #Private data - Encrypted data
    * `checksum` - Checksum of the public data
    * ....
* `removeToken: string` - Write-only token to remove the row from the database

### User ###

* `ID : int` - Unique ID for the user
* `Date : timestamp` - Date when the resource was created
* `Username : string` - String used to easily identify an user
* `StaticPublicKey : PublicKey` - User's FHMQV static public key
* `EphemeralPublicKey : PublicKey` - User's FHMQV ephemeral public key
* `SigningPublicKey : PublicKey` - User's public key used to sign messages
* `Salt : SymmetricKey` - Salt for PBKDF2
* `PublicProfile : int` - Reference to the cleartext user profile
* #PrivateData - Encrypted with user's secret key
    * `Checksum : Hash` - Checksum of all the public data
    * `StaticPrivateKey : PrivateKey` - User's FHMQV static private key
    * `EphemeralPrivateKey : PrivateKey` - User's FHMQV ephemeral private key
    * `SigningPrivateKey : PrivateKey` - User's private key used to sign
                                         messages
    * `LastProcessedMessageID : int`: ID of the last processed message
    * Groups
        * `ID : int` - Group's GroupID
        * `PublicKey : PublicKey` - Hash?
        * `Key : SymmetricKey` - R/W key for the group
        * `AdminKey : SymmetricKey` - Key for the admin data of the group
        * `SigningPrivateKey : PrivateKey` - If administrator
    * Friends
        * `ID : int` - Friend's UserID
        * `PublicKey : PublicKey` - Hash?
        * `Key : SymmetricKey` - For 1-to-1 communications
        * `TrustLevel : int` - Depends on the type of verification of the
                               public key
        * `FriendshipProgress : int` -
        * `LastMessageID : int` - Last message ID of the private conversation
        * `MyLastMessageID : int` - The ID of my last message with this friend
                                     conversation wrongly set by the other end
        * `SMP : Object` - Storage for the SMP protocol
        * `FHMQV : Object` - Storage for the FHMQV protocol
        * Profiles
            * `ID : int` - ID to a profile the user has access to
            * `ProfileKey : SymmetricKey` - Key for the profile
    * Profiles
        * `ID : int` - ID to a profile the user has access to
        * `Key : SymmetricKey` - Key for the profile

### Profile ###

Public profiles can be encrypted null-key.

* `ID : int` - Unique ID for the profile
* `Date : timestamp` - Date when the resource was created
* #$PrivateData - Signed with user's public key, encrypted with a symmetric key
                  which can be distributed 1-to-1 or in a group
    * `UserID : int` - UserID of the owner
    * Groups
        * `ID : int` - ID of the group.
    * `Name : string` - Shown name
    * `Age : int` - Age

### Group ###

* `ID : int` - Unique ID for the group
* `Date : timestamp` - Date when the resource was created
* `SigningPublicKey : PublicKey` - Public key to verify signature
* #$PrivateData - Signed with group's asymmetric key, encrypted with group's
                  symmetric key, if you have the group's private key, you're an
                  admin
    * `Name : string` - Name of the group
    * `FriendsOf : int` - If present, the group is not a standalone group
    *                     but a the group of friends of the specified user
    * `Members : int[]` - User IDs of members allowed to post
        * `ID : int` - Member's UserID
        * `PublicKey : PublicKey` - Hash?
        * `TrustLevel : int` Depends on the type of verification of the public
                             key
    * `Subscribers : int[]` - User IDs of read-only members (eventually)
        * `ID : int` - Subscriber's UserID
        * `PublicKey : PublicKey` - Hash?
        * `TrustLevel : int` Depends on the type of verification of the public
                             key

### Message ###
* `ID : int` - Unique ID for a message
* `Date : timestamp` - Date when the resource was created
* `RecipientType : enum` - Type of the recipient, it can be a group, a profile
                           or an user
* `RecipientID : int` - ID of the recipient
* #$PrivateData - Signed with author's asymmetric key, encrypted with
                  recipient's (user or group) symmetric key
    * `SenderID : int` - fake ID of the sender
    * `Type : string` - Type of the message
        * `Type == "post"`
            * `Post : string` - Content of the post
            * `ReplyTo : int` - If is set the post answers to another post,
                                otherwise it's a normal post
        * `Type == "friendship-message"`
            * ...
        * `Type = "group-joined"`
            * `GroupID : int`
        * `Type = "group-administrator"`
            * `GroupID : int` -
            * `PrivateKey` - Private key of the group

### WoT ###
* `Member : HMAC` - MAC of the member's ID using the group key as key
* `Group : HMAC` - MAC of the group ID using the group key as key
* `editToken: string` - Write-only token to edit the row in the database
